#include <bitmapfonts.c>
#include <fonts/font.dat.c>
#include <gl/shaderprocs.c>
#include <config.c>
#include <rnd/uc.c>
#include <rnd/sq.c>
#include <rnd/rnd.c>
#include <rnd/frnd.c>

char *fragprog=(char *)"\n\
uniform float t;\n\
const int M=%d;\n\
uniform float xres,yres;\n\
uniform int m;\n\
uniform float rr[M],gg[M],bb[M],gs[M],gi[M],cx[M],cy[M],cgr[M],sgr[M];\n\
uniform int gt[M];\n\
float sq(const float x) {return x*x;}\n\
void main()\n\
{\n\
	float x=gl_TexCoord[0].x,y=gl_TexCoord[0].y;\n\
	float r=0,g=0,b=0,ii,xx,yy;\n\
	for (int n=0;n<m;n++)\n\
	{\n\
		xx=(cgr[n]*(x-xres/2)+sgr[n]*(y-yres/2)-cx[n])/gs[n];\n\
		yy=(cgr[n]*(y-yres/2)-sgr[n]*(x-xres/2)-cy[n])/gs[n];\n\
		if (gt[n]&2) {xx/=yy; yy=1.0/yy;}\n\
		if (gt[n]&8) {float dd=xx*xx+yy*yy; xx/=dd; yy/=dd;}\n\
		ii=gi[n]/(sq(mod(xx+10000+t,1)-0.5)+sq(mod(yy+10000+t*0.3,1)-0.5));\n\
		if (gt[n]&1)\n\
		{\n\
			r=1.0-(1.0-r)*pow(1.0-rr[n],ii);\n\
			g=1.0-(1.0-g)*pow(1.0-gg[n],ii);\n\
			b=1.0-(1.0-b)*pow(1.0-bb[n],ii);\n\
		}\n\
		else\n\
		{\n\
			r*=pow(rr[n],ii);\n\
			g*=pow(gg[n],ii);\n\
			b*=pow(bb[n],ii);\n\
		}\n\
	}\n\
	gl_FragColor.r=r;\n\
	gl_FragColor.g=g;\n\
	gl_FragColor.b=b;\n\
	gl_FragColor.a=1;\n\
}";

#define F(n) (float *)malloc((n)*sizeof(float))

WINMAIN
{
	LSet conf=configload("prefs.txt");
	configdefault(&conf,"Complexity","25");
	configdefault(&conf,"Seconds between images","7.5");
	configdefault(&conf,"Animation speed","10");
	int n,x,y,m,N=configgeti(&conf,"Complexity");
	char *fragprogN=(char *)malloc(strlen(fragprog)+20);
	sprintf(fragprogN,fragprog,N);
	double speed=configgetd(&conf,"Animation speed");
	double sleepperiod=configgetd(&conf,"Seconds between images");
	float *rr=F(N),*gg=F(N),*bb=F(N),*gs=F(N),*gr=F(N),*gi=F(N),*cx=F(N),*cy=F(N),scale;
	float *cgr=F(N),*sgr=F(N);
	int *gt=(int *)malloc(N*sizeof(int));
	srand(GetTickCount());
	GRAPHICS(0,0,"Lights and Perspective OpenGL version [Stephen Brooks 2020]");
	loadfontresource(font_dat,"normal"); char str[200];
	char *slog,*plog;
	GLuint prog=shaderprogfrag(fragprogN,&slog,&plog);
	scale=(xres+yres)/2;
	int genparams=1; double tgen=eithertime();
	LOOP
	{
		if (eithertime()>=tgen) genparams=1;
		if (genparams) // Generate params
		{
			m=1+rnd(N);
			for (n=0;n<m;n++)
			{
				rr[n]=frnd(1); gg[n]=frnd(1); bb[n]=frnd(1);
				gs[n]=pow(scale,frnd(1)); gr[n]=frnd(M_TWOPI);
				gi[n]=0.01;
				gt[n]=rnd(32);
				cx[n]=frnd(scale)-scale/2; cy[n]=frnd(scale)-scale/2;
				cgr[n]=cos(gr[n]); sgr[n]=sin(gr[n]);
			}
			x=1;
			while (x)
			{
				x=0;
				for (n=0;n+1<m;n++) if (gs[n+1]<gs[n])
				{
					float f=gs[n]; gs[n]=gs[n+1]; gs[n+1]=f;
					x=1;
				}
			}
			genparams=0;
			tgen=eithertime()+sleepperiod;
		}
		glUseProgram(prog);
		glUniform1f(glGetUniformLocation(prog,"t"),fmod(eithertime()*speed/10,10));
		glUniform1f(glGetUniformLocation(prog,"xres"),xres);
		glUniform1f(glGetUniformLocation(prog,"yres"),yres);
		glUniform1i(glGetUniformLocation(prog,"m"),m);
		glUniform1fv(glGetUniformLocation(prog,"rr"),m,rr);
		glUniform1fv(glGetUniformLocation(prog,"gg"),m,gg);
		glUniform1fv(glGetUniformLocation(prog,"bb"),m,bb);
		glUniform1fv(glGetUniformLocation(prog,"gs"),m,gs);
		glUniform1fv(glGetUniformLocation(prog,"gi"),m,gi);
		glUniform1fv(glGetUniformLocation(prog,"cx"),m,cx);
		glUniform1fv(glGetUniformLocation(prog,"cy"),m,cy);
		glUniform1fv(glGetUniformLocation(prog,"cgr"),m,cgr);
		glUniform1fv(glGetUniformLocation(prog,"sgr"),m,sgr);
		glUniform1iv(glGetUniformLocation(prog,"gt"),m,gt);
		if (!KEY(VK_D))
		{
			glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex2i(0,0);
			glTexCoord2f(xres,0); glVertex2i(xres,0);
			glTexCoord2f(xres,yres); glVertex2i(xres,yres);
			glTexCoord2f(0,yres); glVertex2i(0,yres);
			glEnd();
		}
		glUseProgram(0);
		if (KHIT(VK_S))
		{
			sprintf(str,"scr%X.bmp",GetTickCount());
			screenshot(str,0);
		}
		if (KEY(VK_D))
		{
			sprintf(str,"prog=%d glGetError()=%d (GL_NO_ERROR=%d)",
				prog,glGetError(),GL_NO_ERROR);
			bfwrite(0,0,str,GFX_white);
			sprintf(str,"slog='%s'",slog); bfwrite(xres/3,0,str,GFX_white);
			sprintf(str,"plog='%s'",plog); bfwrite(xres*2/3,0,str,GFX_white);
		}
		ccl();
	}
	configfree(&conf); free(fragprogN); free(slog); free(plog);
}
